@@ -1,3 +1,5 @@
+import os
+
 from .norm import SELECT
 from .norm import UPDATE
 from .norm import INSERT
@@ -8,6 +10,11 @@ from norm.connection import ConnectionProxy
 from norm.connection import ConnectionFactory
 from norm.connection import CursorProxy
 
+enable_logging = False
+if int(os.getenv('NORM_LOG_QUERIES', 0)):
+    enable_logging = True
+
+
 __all__ = ['SELECT',
            'UPDATE',
            'DELETE',
