@@ -7,6 +7,7 @@ WHERE = b'w'
 HAVING = b'h'
 GROUP_BY = b'gb'
 ORDER_BY = b'ob'
+TOP = b'top'
 LIMIT = b'l'
 OFFSET = b'os'
 EXTRA = b'ex'
@@ -44,6 +45,10 @@ class NormAsIs:
         except AttributeError:
             return False
 
+    @classmethod
+    def isasis(cls, o):
+        return isinstance(o, cls)
+
 
 class BogusQuery(Exception):
     pass
@@ -62,6 +67,7 @@ def compile(chain, query_type):
     having = []
     group_by = []
     order_by = []
+    top = None
     limit = None
     offset = None
     set_ = []
@@ -91,6 +97,8 @@ def compile(chain, query_type):
             group_by.append(option)
         elif op == ORDER_BY:
             order_by.append(option)
+        elif op == TOP:
+            top = option
         elif op == LIMIT:
             limit = option
         elif op == OFFSET:
@@ -106,7 +114,12 @@ def compile(chain, query_type):
 
     query = ''
     if query_type == SELECT_QT:
-        query += 'SELECT ' + COLUMN_SEP.join(columns)
+        query += 'SELECT '
+        if top is not None:
+            query += 'TOP ' + top + SEP
+
+        query += COLUMN_SEP.join(columns)
+
         if from_:
             query += '\n  FROM ' + SEP.join(from_)
         if where:
@@ -159,8 +172,14 @@ class Query:
         self._binds = []
         self._query = None
 
+    @classmethod
+    def clean_bind_name(cls, s):
+        s = s.replace('.', '___')
+        return s
+
     @classmethod
     def bnd(cls, s):
+        s = cls.clean_bind_name(s)
         return "%s%s%s" % (cls.bind_prefix, s, cls.bind_postfix)
 
     @property
@@ -180,17 +199,21 @@ class Query:
 
     @property
     def binds(self):
-        return dict(self.bind_items)
-        # binds = {}
-        # if self.parent is not None:
-        #     binds.update(self.parent.binds)
-        # binds.update(self._binds)
-
-        # return binds
+        operative_binds = {}
+        for key, value in self.bind_items:
+            if NormAsIs.isasis(value):
+                operative_binds.pop(key, None)
+                continue
+            operative_binds[key] = value
+        return operative_binds
 
     def bind(self, **binds):
         s = self.child()
-        s._binds = list(binds.items())
+        final_binds = []
+        for name, value in binds.items():
+            name = self.clean_bind_name(name)
+            final_binds.append((name, value))
+        s._binds = final_binds
         return s
 
     def child(self):
@@ -209,7 +232,23 @@ class Query:
     def query(self):
         if self._query is None:
             self._query = compile(self.build_chain(), self.query_type)
-        return self._query
+        query = self._query
+
+        final_binds = {}
+        for key, value in self.bind_items:
+            final_binds[key] = value
+        for key in sorted(final_binds.keys(), key=len, reverse=True):
+            value = final_binds[key]
+            if NormAsIs.isasis(value):
+                query = query.replace(self.bnd(key), value.value)
+        return query
+
+    @property
+    def _loggable_query(self):
+        query = self.query
+        for key, value in self.binds.items():
+            query = query.replace(self.bnd(key), repr(value))
+        return query
 
 
 class _SELECT_UPDATE(Query):
@@ -219,7 +258,8 @@ class _SELECT_UPDATE(Query):
         for stmt in args:
             s.chain.append((WHERE, stmt))
         for column_name, value in kw.items():
-            bind_val_name = '%s_bind_%s' % (column_name, s.bind_len)
+            bind_val_name = '%s_bind_%s' % (
+                self.clean_bind_name(column_name), s.bind_len)
             s._binds.append((bind_val_name, value))
             expr = column_name + ' = ' + self.bnd(bind_val_name)
             s.chain.append((WHERE, expr))
@@ -294,6 +334,13 @@ class SELECT(_SELECT_UPDATE):
             s.chain.append((GROUP_BY, arg))
         return s
 
+    def TOP(self, stmt):
+        if isinstance(stmt, int):
+            stmt = str(stmt)
+        s = self.child()
+        s.chain.append((TOP, stmt))
+        return s
+
     def LIMIT(self, stmt):
         if isinstance(stmt, int):
             stmt = str(stmt)
@@ -324,7 +371,8 @@ class UPDATE(_SELECT_UPDATE):
             self.chain.append((SET, stmt))
 
         for column_name, value in kw.items():
-            bind_name = column_name + '_bind'
+            clean_column_name = self.clean_bind_name(column_name)
+            bind_name = clean_column_name + '_bind'
             s._binds.append((bind_name, value))
             expr = str(column_name) + ' = ' + self.bnd(bind_name)
             s.chain.append((SET, expr))
@@ -344,11 +392,11 @@ class DELETE(_SELECT_UPDATE):
             self.chain.append((TABLE, table))
 
 
-class _default(object):
+class _default:
     pass
 
 
-class INSERT(object):
+class INSERT:
     bind_prefix = '%('
     bind_postfix = ')s'
     defaultdefault = None
@@ -394,10 +442,11 @@ class INSERT(object):
                 key = f'{col_name}_{index}'
 
                 if col_name in d:
-                    if not self._is_asis(d[col_name]):
+                    if not NormAsIs.isasis(d[col_name]):
                         binds[key] = d[col_name]
                 else:
-                    binds[key] = self.default
+                    if not NormAsIs.isasis(self.default):
+                        binds[key] = self.default
 
         return binds
 
@@ -431,34 +480,30 @@ class INSERT(object):
     def _bind_param_name(self, col_name, index):
         return f'{self.bind_prefix}{col_name}_{index}{self.bind_postfix}'
 
-    def _is_asis(self, val):
-        if isinstance(val, NormAsIs):
-            return True
-        return False
-
     def _query(self, data):
         q = 'INSERT INTO %s ' % self.table
 
         if self.columns:
             q += '('
             q += ', '.join(col_name for col_name in self.columns)
-            q += ') '
+            q += ')'
 
         if self.statement:
             q += '\n' + indent_string(self.statement.query[:-1], 2)
         elif self.data is None:
             q += 'DEFAULT VALUES'
         else:
-            q += 'VALUES '
+            q += '\n  VALUES\n'
             for index, d in enumerate(data):
                 if index > 0:
-                    q += ',\n       '
+                    q += ',\n'
 
                 q += '('
                 last_col_ix = len(self.columns) - 1
                 for ix, col_name in enumerate(self.columns):
-                    if self._is_asis(d.get(col_name)):
-                        q += d.get(col_name).value
+                    col_val = d.get(col_name, self.default)
+                    if NormAsIs.isasis(col_val):
+                        q += col_val.value
                     else:
                         q += self._bind_param_name(col_name, index)
                     if last_col_ix != ix:
@@ -495,13 +540,12 @@ class WITH(Query):
         parts = []
         for name, query in self.tables.items():
             query_part = query.query[:-1]  # TODO: HACK!
-            query_section = indent_string(f'({query_part})\n', 7)
+            query_section = indent_string(f'({query_part})\n', 8)[1:]
             parts.append(f'{name} AS\n{query_section}')
         return ('WITH ' +
                 ',\n     '.join(parts) +
                 '\n\n' +
-                self.primary.query +
-                ';')
+                self.primary.query)
 
     @property
     def binds(self):
@@ -518,4 +562,5 @@ __all__ = [Query,
            UPDATE,
            DELETE,
            INSERT,
+           WITH,
            BogusQuery]
